{ Convert functions
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _conv;

interface

uses
 _common;


var
 byt               : array[0..10] of byte;


procedure ResetBytes;

procedure ValueToBytes(v: TAsmValue);
function BytesToValue(by1,by2,by3,by4,by5,by6,by7: byte): TAsmValue;

function ByteToHex(b: byte): string;
function DwordToHex(d: TDefValue): string;

function GetHexValue(c: char): byte;

function HexToInt(h: string): TAsmValue;
function BinToInt(b: string): TAsmValue;
function OctToInt(o: string): TAsmValue;

function DigitalNum(val: TDefValue; chr: char; cnt: integer): string;

implementation

uses
 SysUtils;


procedure ResetBytes;
var
 i                 : integer;
begin
   for i:= 1 to 10 do byt[i]:= 0;
end;



procedure ValueToBytes(v: TAsmValue);
var
 vv                : TAsmValue;
begin
   vv:= v;
   ResetBytes;
   if vv >= 0 then begin
      byt[7]:= vv div 281474976710656;
      vv:= vv - (byt[7] * 281474976710656);
      byt[6]:= vv div 1099511627776;
      vv:= vv - (byt[6] * 1099511627776);
      byt[5]:= vv div 4294967296;
      vv:= vv - (byt[5] * 4294967296);
      byt[4]:= vv div 16777216;
      vv:= vv - (byt[4] * 16777216);
      byt[3]:= vv div 65536;
      vv:= vv - (byt[3] * 65536);
      byt[2]:= vv div 256;
      vv:= vv - (byt[2] * 256);
      byt[1]:= vv;
   end else begin
      vv:= -1 - v;

      byt[10]:= 255;
      byt[9]:= 255;
      byt[8]:= 255;

      byt[7]:= vv div 281474976710656;
      vv:= vv - (byt[7] * 281474976710656);
      byt[7]:= 255 - byt[7];
      byt[6]:= vv div 1099511627776;
      vv:= vv - (byt[6] * 1099511627776);
      byt[6]:= 255 - byt[6];
      byt[5]:= vv div 4294967296;
      vv:= vv - (byt[5] * 4294967296);
      byt[5]:= 255 - byt[5];
      byt[4]:= vv div 16777216;
      vv:= vv - (byt[4] * 16777216);
      byt[4]:= 255 - byt[4];
      byt[3]:= vv div 65536;
      vv:= vv - (byt[3] * 65536);
      byt[3]:= 255 - byt[3];
      byt[2]:= vv div 256;
      vv:= vv - (byt[2] * 256);
      byt[2]:= 255 - byt[2];
      byt[1]:= 255 - vv;
   end;
end;

function BytesToValue(by1,by2,by3,by4,by5,by6,by7: byte): TAsmValue;
begin
   BytesToValue:= (by7 * 281474976710656) +
   (by6 * 1099511627776) + (by5 * 4294967296) +
   (by4 * 16777216) + (by3 * 65536) + (by2 * 256) + by1;
end;



function ByteToHex(b: byte): string;
const
 hexTable          = '0123456789ABCDEF';
var
 hStr              : string;
 bb, bHigh, bLow   : byte;
begin
   hStr:= '--';
   bb:= b;
   bHigh:= bb div 16;
   bLow:= bb - (bHigh * 16);
   hStr:= Copy(hexTable, bHigh+1, 1) + Copy(hexTable, bLow+1, 1);
   ByteToHex:= hStr;
end;

function DwordToHex(d: TDefValue): string;
var
 dStr              : string;
begin
   ValueToBytes(d);
   dStr:= ByteToHex(byt[4]) + ByteToHex(byt[3]) +
    ByteToHex(byt[2]) + ByteToHex(byt[1]);
   DwordToHex:= dStr;
end;



function GetHexValue(c: char): byte;
var
 rV                 : byte;
begin
   rV:= 0;
   if (c in ['0'..'9']) then
    rV:= ord(c) - ord('0')
   else begin
      if UpCase(c) = 'A' then rV:= 10;
      if UpCase(c) = 'B' then rV:= 11;
      if UpCase(c) = 'C' then rV:= 12;
      if UpCase(c) = 'D' then rV:= 13;
      if UpCase(c) = 'E' then rV:= 14;
      if UpCase(c) = 'F' then rV:= 15;
   end;
   GetHexValue:= rV;
end;



function HexToInt(h: string): TAsmValue;
var
 f, v              : TAsmValue;
 i                 : integer;
begin
   f:= 1;
   v:= 0;
   if Length(h) > 0 then begin
      for i:= Length(h) downto 1 do begin
         v:= v + (f * GetHexValue(h[i]));
         f:= f * 16;
      end;
   end;
   HexToInt:= v;
end;

function BinToInt(b: string): TAsmValue;
var
 f, v              : TAsmValue;
 i                 : integer;
begin
   f:= 1;
   v:= 0;
   if Length(b) > 0 then begin
      for i:= Length(b) downto 1 do begin
         v:= v + (f * (ord(b[i]) - ord('0')));
         f:= f * 2;
      end;
   end;
   BinToInt:= v;
end;

function OctToInt(o: string): TAsmValue;
var
 f, v              : TAsmValue;
 i                 : integer;
begin
   f:= 1;
   v:= 0;
   if Length(o) > 0 then begin
      for i:= Length(o) downto 1 do begin
         v:= v + (f * (ord(o[i]) - ord('0')));
         f:= f * 8;
      end;
   end;
   OctToInt:= v;
end;



function DigitalNum(val: TDefValue; chr: char; cnt: integer): string;
var
 dNum              : string;
 i, ln             : integer;
begin
   dNum:= IntToStr(val);
   ln:= Length(dNum);
   if ln < cnt then begin
      for i:= ln+1 to cnt do begin
         dNum:= chr + dNum;
      end;
   end else begin
      if ln > cnt then begin
         dNum:= Copy(dNum, 1, cnt);
      end;
   end;
   DigitalNum:= dNum;
end;



end.
