{ Disk I/O
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _diskio;

interface

var
 fff               : file of byte;



procedure WriteByte(b: byte);
procedure WriteWord(w: word);
procedure WriteDword(d: cardinal);
procedure WriteQword(q: int64);
procedure WriteString(s: string);
procedure WriteRAWStr(s: string);

function ReadByte: byte;
function ReadWord: word;
function ReadDword: cardinal;
function ReadQword: int64;
function ReadString: string;
function ReadStringN(ln: integer): string;

function ExtFileDir(filename: string): string;
function RemoveExt(filename: string): string;
function RemoveExt2(filename, ext: string): string;
function ToCmdFile(filname: string; ch: char): string;

function CreateMultiDir(path: string): boolean;

implementation

uses
 SysUtils, _common, _conv;

procedure WriteByte(b: byte);
begin
   write(fff, b);
end;

procedure WriteWord(w: word);
begin
   ValueToBytes(w);
   write(fff, byt[1]);
   write(fff, byt[2]);
end;

procedure WriteDword(d: cardinal);
begin
   ValueToBytes(d);
   write(fff, byt[1]);
   write(fff, byt[2]);
   write(fff, byt[3]);
   write(fff, byt[4]);
end;

procedure WriteQword(q: int64);
begin
   ValueToBytes(q);
   write(fff, byt[1]);
   write(fff, byt[2]);
   write(fff, byt[3]);
   write(fff, byt[4]);
   write(fff, byt[5]);
   write(fff, byt[6]);
   write(fff, byt[7]);
   write(fff, byt[8]);
end;

procedure WriteString(s: string);
var
 l                 : byte;
 b                 : byte;
 i                 : integer;
begin
   l:= length(s);
   write(fff, l);
   if l > 0 then begin
      for i:= 1 to l do begin
         b:= ord(s[i]);
         write(fff, b);
      end;
   end;
end;

procedure WriteRAWStr(s: string);
var
 l                 : byte;
 b                 : byte;
 i                 : integer;
begin
   l:= length(s);
   if l > 0 then begin
      for i:= 1 to l do begin
         b:= ord(s[i]);
         write(fff, b);
      end;
   end;
end;



function ReadByte: byte;
var
 c                 : byte;
begin
   read(fff, c);
   ReadByte:= c;
end;

function ReadWord: word;
begin
   byt[1]:= ReadByte;
   byt[2]:= ReadByte;
   ReadWord:= BytesToValue(byt[1], byt[2], 0, 0, 0, 0, 0);
end;

function ReadDword: cardinal;
begin
   byt[1]:= ReadByte;
   byt[2]:= ReadByte;
   byt[3]:= ReadByte;
   byt[4]:= ReadByte;
   ReadDword:= BytesToValue(byt[1], byt[2], byt[3], byt[4], 0, 0, 0);
end;

function ReadQword: int64;
begin
   byt[1]:= ReadByte;
   byt[2]:= ReadByte;
   byt[3]:= ReadByte;
   byt[4]:= ReadByte;
   byt[5]:= ReadByte;
   byt[6]:= ReadByte;
   byt[7]:= ReadByte;
   byt[8]:= ReadByte;
   ReadQword:= BytesToValue(byt[1], byt[2], byt[3], byt[4], byt[5], byt[6], byt[7]);
end;

function ReadString: string;
var
 l                 : byte;
 i                 : integer;
 c                 : byte;
 s                 : string;
begin
   l:= ReadByte;
   s:= '';
   if l > 0 then begin
      for i:= 1 to l do begin
         c:= ReadByte;
         s:= s + chr(c);
      end;
   end;
   ReadString:= s;
end;

function ReadStringN(ln: integer): string;
var
 i                 : integer;
 s                 : string;
begin
   s:= '';
   if ln > 0 then begin
      for i:= 1 to ln do s:= s + chr(ReadByte);
   end;
   ReadStringN:= s;
end;



function ExtFileDir(filename: string): string;
var
  pos              : integer;
  newPath          : string;
begin
   pos:= Length(filename);
   newPath:= '';
   if pos > 0 then begin
      while true do begin
         if pos = 1 then break;
         if (filename[pos] = '\') or (filename[pos] = '/')
          or (filename[pos] = ':') then begin
            Dec(pos);
            if pos > 0 then newPath:= Copy(filename, 1, pos);
            break;
         end;
         Dec(pos);
      end;
   end;
   ExtFileDir:= newPath;
end;

function RemoveExt(filename: string): string;
var
  pos              : integer;
  newPath          : string;
begin
   pos:= Length(filename);
   newPath:= filename;
   if pos > 0 then begin
      while true do begin
         if pos = 1 then break;
         if filename[pos] = '.' then begin
            Dec(pos);
            if pos > 0 then newPath:= Copy(filename, 1, pos);
            break;
         end;
         Dec(pos);
      end;
   end;
   RemoveExt:= newPath;
end;

function RemoveExt2(filename, ext: string): string;
var
  pos              : integer;
  newPath          : string;
  curExt           : string;
begin
   pos:= Length(filename);
   newPath:= filename;
   if pos > 0 then begin
      while true do begin
         if pos = 1 then break;
         if filename[pos] = '.' then begin
            Dec(pos);
            if pos > 0 then begin
               curExt:= Copy(filename, pos+2, Length(filename)-(pos+1));
               if UpperCase(curExt) = UpperCase(ext) then begin
                  newPath:= Copy(filename, 1, pos);
               end else begin
                  newPath:= filename;
               end;
            end;
            break;
         end;
         Dec(pos);
      end;
   end;
   RemoveExt2:= newPath;
end;

function ToCmdFile(filname: string; ch: char): string;
var
 nf                : string;
 i                 : integer;
begin
   nf:= '';
   if length(filname) > 0 then begin
      for i:= 1 to length(filname) do begin
         if filname[i] = ' ' then
          nf:= nf + ch
         else
          nf:= nf + filname[i];
      end;
   end;
   ToCmdFile:= nf;
end;



function CreateMultiDir(path: string): boolean;
var
 okStat            : boolean;
 p, l              : integer;
 cpath             : string;
begin
   okStat:= false;
   l:= length(path);
   p:= 1;
   if l > 3 then begin
      if (path[2] = ':') and ((path[3] = '\') or (path[3] = '/')) then begin
         p:= 4;
      end;
   end;
   while p <= l do begin
      if (path[p] = '\') or (path[p] = '/') or (p = l) then begin
         if p > 1 then begin
            if p = l then
             cpath:= copy(path, 1, p)
            else
             cpath:= copy(path, 1, p-1); 
            if DirectoryExists(cpath) then begin
               okStat:= true;
            end else begin
               {$I-}
               MkDir(cpath);
               {$I+}
               if IOResult = 0 then
                okStat:= true
               else begin
                  okStat:= false;
                  break;
               end;
            end;
         end;
      end;
      inc(p);
   end;
   CreateMultiDir:= okStat;
end;



end.
