{ Define (Dx = DB, DW, DD, DQ, DF, DP, DT)
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _dx;

interface

uses
 _common;

procedure BuildDataString(mmType: byte; ssOfs: TDefValue; ssLen: TAsmValue);
procedure GrabDataString(mmType: byte);
procedure DefineMemData(n: string; mType: byte);  

procedure DefineArray(n: string);

implementation

uses
 SysUtils, _error, _mem, _codegen, _scanner, _parser,
 _source, _ident, _seg;

procedure BuildDataString(mmType: byte; ssOfs: TDefValue; ssLen: TAsmValue);
var
 memType, b, t     : byte;
 i, id             : TDefValue;
 memSize           : TDefValue;
 strOfs            : TDefValue;
 strLen            : TAsmValue;
 totalLen          : TAsmValue;
 sName, iStr, gStr : string;
 eFnd              : boolean;
 val               : TAsmValue;
 sStr              : string;
 numItems, fBytes  : TAsmValue;
 oBytes            : TAsmValue;
begin
   memType:= mmType;
   strOfs:= ssOfs;
   strLen:= ssLen;
   totalLen:= ssLen;

   memSize:= 1;
   if memType = tWORD then memSize:= 2;
   if memType = tDWORD then memSize:= 4;
   if memType = tQWORD then memSize:= 8;
   if memType = tFARPTR then memSize:= 6;
   if memType = tTENBYTE then memSize:= 10;

   if strLen > 0 then begin
      i:= 0;
      while i < strLen do begin
         b:= MemData[strOfs + i];
         AddCodeListValue(tBYTE, b);
         Inc(i);
      end;
   end;

   if not IsErr then begin
      while true do begin
         if IsErr then break;
         SkipBlank;
         if GetSrcChar = '+' then begin
            NextChar;
            SkipBlank;
            sName:= GetName;
            if not IsErr then begin
               if sName <> '' then begin
                  id:= GetIdentIndex(sName);
                  if id = NotFound then begin
                     ErrorMsg('Identifier not found [' + sName + ']!');
                     break;
                  end else begin
                     if IdentList[id].IdentType = itEQU then begin
                        t:= IdentList[id].EquMode;
                        eFnd:= false;
                        if t = equModeINT then begin
                           eFnd:= true;
                           iStr:= IntToStr(IdentList[id].IValue);
                           strLen:= Length(iStr);
                           if strLen > 0 then begin
                              i:= 1;
                              while i <= strLen do begin
                                 b:= ord(iStr[i]);
                                 AddCodeListValue(tBYTE, b);
                                 Inc(i);
                              end;
                           end;
                           totalLen:= totalLen + strLen;
                        end;
                        if t = equModeSTR then begin
                           eFnd:= true;
                           strOfs:= IdentList[id].SValue;
                           strLen:= IdentList[id].StrLen;
                           if strLen > 0 then begin
                              i:= 0;
                              while i < strLen do begin
                                 b:= MemData[strOfs + i];
                                 AddCodeListValue(tBYTE, b);
                                 Inc(i);
                              end;
                           end;
                           totalLen:= totalLen + strLen;
                        end;
                        if not eFnd then begin
                           ErrorMsg('Incompatible equate type [' + sName + ']!');
                        end;
                        if IsErr then break;
                     end else begin
                        ErrorMsg('Identifier is not an equate [' + sName + ']!');
                        break;
                     end;
                  end;
               end else begin
                  if (GetSrcChar = '"') or (GetSrcChar = #39) then begin
                     gStr:= GrabSimpleString;
                     if not IsErr then begin
                        strLen:= Length(gStr);
                        if strLen > 0 then begin
                           i:= 1;
                           while i <= strLen do begin
                              b:= ord(gStr[i]);
                              AddCodeListValue(tBYTE, b);
                              Inc(i);
                           end;
                        end;
                        totalLen:= totalLen + strLen;
                     end;
                  end else begin
                     val:= GetNum;
                     if not IsErr then begin
                        sStr:= IntToStr(val);
                        strLen:= Length(sStr);
                        if strLen > 0 then begin
                           i:= 1;
                           while i <= strLen do begin
                              b:= ord(sStr[i]);
                              AddCodeListValue(tBYTE, b);
                              Inc(i);
                           end;
                        end;
                        totalLen:= totalLen + strLen;
                     end;
                  end;
               end;
            end;
         end else begin
            break;
         end;
      end;
   end;

   if not IsErr then begin
      numItems:= totalLen div memSize;
      fBytes:= memSize;
      if memSize > 1 then begin
         if totalLen > 0 then begin
            oBytes:= totalLen - (numItems * memSize);
            if (oBytes > 0) or (oBytes < memSize) then begin
               fBytes:= memSize - oBytes;
            end else begin
               fBytes:= 0;
            end;
         end;
         if fBytes > 0 then begin
            i:= 0;
            while i < fBytes do begin
               AddCodeListValue(tBYTE, 0);
               Inc(i);
            end;
         end;
      end;
   end;
end;

procedure GrabDataString(mmType: byte);
var
 memType, b, t     : byte;
 memSize, i, id    : TDefValue;
 totalLen          : TAsmValue;
 dStr, sName       : string;
 strOfs            : TDefValue;
 strLen            : TAsmValue;
 eFnd              : boolean;
 iStr, gStr, sStr  : string;
 val               : TAsmValue;
 numItems, fBytes  : TAsmValue;
 oBytes            : TAsmValue;
begin
   memType:= mmType;
   totalLen:= 0;

   memSize:= 1;
   if memType = tWORD then memSize:= 2;
   if memType = tDWORD then memSize:= 4;
   if memType = tQWORD then memSize:= 8;
   if memType = tFARPTR then memSize:= 6;
   if memType = tTENBYTE then memSize:= 10;
   
   dStr:= GrabSimpleString;
   { strOfs:= 0; }
   strLen:= Length(dStr);

   if not IsErr then begin
      i:= 1;
      while i <= strLen do begin
         b:= ord(dStr[i]);
         AddCodeListValue(tBYTE, b);
         Inc(i);
      end;
      totalLen:= strLen;
	  
      while true do begin
         if IsErr then break;
         SkipBlank;
         if GetSrcChar = '+' then begin
            NextChar;
            SkipBlank;
            sName:= GetName;
            if not IsErr then begin
               if sName <> '' then begin
                  id:= GetIdentIndex(sName);
                  if id = NotFound then begin
                     ErrorMsg('Identifier not found [' + sName + ']!');
                     break;
                  end else begin
                     if IdentList[id].IdentType = itEQU then begin
                        t:= IdentList[id].EquMode;
                        eFnd:= false;
                        if t = equModeINT then begin
                           eFnd:= true;
                           iStr:= IntToStr(IdentList[id].IValue);
                           strLen:= Length(iStr);
                           if strLen > 0 then begin
                              i:= 1;
                              while i <= strLen do begin
                                 b:= ord(iStr[i]);
                                 AddCodeListValue(tBYTE, b);
                                 Inc(i);
                              end;
                           end;
                           totalLen:= totalLen + strLen;
                        end;
		                   	if t = equModeSTR then begin
                           eFnd:= true;
                           strOfs:= IdentList[id].SValue;
                           strLen:= IdentList[id].StrLen;
                           if strLen > 0 then begin
                              i:= 0;
                              while i < strLen do begin
                                 b:= MemData[strOfs + i];
                                 AddCodeListValue(tBYTE, b);
                                 Inc(i);
                              end;
                           end;
                           totalLen:= totalLen + strLen;
                        end;
                        if not eFnd then begin
                           ErrorMsg('Incompatible equate type [' + sName + ']!');
                        end;
                        if IsErr then break;
                     end else begin
                        ErrorMsg('Identifier is not an equate [' + sName + ']!');
                        break;
                     end;
                  end;
               end else begin
                  if (GetSrcChar = '"') or (GetSrcChar = #39) then begin
                     gStr:= GrabSimpleString;
                     if not IsErr then begin
                        strLen:= Length(gStr);
                        if strLen > 0 then begin
                           i:= 1;
                           while i <= strLen do begin
                              b:= ord(gStr[i]);
                              AddCodeListValue(tBYTE, b);
                              Inc(i);
                           end;
                        end;
                        totalLen:= totalLen + strLen;
                     end;
                  end else begin
                     val:= GetNum;
                     if not IsErr then begin
                        sStr:= IntToStr(val);
                        strLen:= Length(sStr);
                        i:= 1;
                        while i <= strLen do begin
                           b:= ord(sStr[i]);
                           AddCodeListValue(tBYTE, b);
                           Inc(i);
                        end;
                        totalLen:= totalLen + strLen;
                     end;
                  end;
               end;
            end;
         end else begin
            break;
         end;
      end;
   end;
   
   if not IsErr then begin
      numItems:= totalLen div memSize;
      fBytes:= memSize;
      if memSize > 1 then begin
         if totalLen > 0 then begin
            oBytes:= totalLen - (numItems * memSize);
            if (oBytes > 0) or (oBytes < memSize) then begin
               fBytes:= memSize - oBytes;
            end else begin
               fBytes:= 0;
            end;
         end;
         if fBytes > 0 then begin
            i:= 0;
            while i < fBytes do begin
               AddCodeListValue(tBYTE, 0);
               Inc(i);
            end;
         end;
      end;
   end;
end;

procedure DefineMemData(n: string; mType: byte);
var
 nam, nam2, enam   : string;
 nFnd              : boolean;
 v, val, nVal      : TAsmValue;
 id, i, segID, eid : TDefValue;
 sOfs              : TDefValue;
 sLen              : TAsmValue;
 t, opr            : byte;
 num               : TAsmValue;
 siz               : byte;
 remPos            : TAsmValue;
begin
   remPos:= GetSegOffset(true);
   if not IsErr then begin
      SkipBlank;
      while true do begin
         nam:= GetName;
         if not IsErr then begin
            if nam <> '' then begin
               nFnd:= false;
               { "DUP": }
               { syntax: DUP data:count }

               if (nam = 'OFFSET') or (nam = 'SIZE') then begin
                  nFnd:= true;
                  nam2:= GetName;
                  if not IsErr then begin
                     if nam2 <> '' then begin
                        if nam = 'OFFSET' then begin
                           v:= GetCalc;
                           if not IsErr then begin
                              AddCodeListEQUOFFSET(mType, nam2, RetOpr, v);
                           end;
                        end else begin
                           v:= GetCalc;
                           if not IsErr then begin
                              AddCodeListEQUSIZE(mType, nam2, RetOpr, v);
                           end;
                        end;
                     end else begin
                        ErrorMsg('Name expected after [' + nam + ']!');
                     end;
                  end;
               end;
	             if not nFnd then begin
                  id:= GetIdentIndex(nam);
                  if id = NotFound then begin
                     v:= GetCalc;
                     if not IsErr then begin
                        segID:= NotFound;
                        if SegCount > 0 then begin
                           i:= 0;
                           while i < SegCount do begin
                              if Segment[i].IdentName = nam then begin
                                 segID:= i;
                                 break;
                              end;
                              Inc(i);
                           end;
                        end;
                        if segID <> NotFound then begin
                           AddCodeListSEGOFFSET(mType, segID, RetOpr, v);
                        end else begin
                           AddCodeListMEM(mType, nam, RetOpr, v);
                        end;
                     end;
                  end else begin
                     if IdentList[id].IdentType = itEQU then begin
                        if IdentList[id].EquMode = equModeSTR then begin
                           sOfs:= IdentList[id].SValue;
                           sLen:= IdentList[id].StrLen;
                           BuildDataString(mType, sOfs, sLen);
                        end else begin
                           v:= GetCalc;
                           if not IsErr then begin
			                        t:= IdentList[id].EquMode;
                              nFnd:= false;
                              if t = equModeINT then begin
                                 nFnd:= true;
                                 val:= IdentList[id].IValue;
                                 val:= DoCalc(val, RetOpr, v);
                                 AddCodeListValue(mType, val);
                              end;
                              if t = equModeOFFSET then begin
                                 nFnd:= true;
                                 val:= IdentList[id].IValue;
                                 opr:= IdentList[id].EquOpr;
                                 eid:= IdentList[id].EquIdentIndex;
                                 enam:= IdentList[id].EquName;
                                 if eid <> NotFound then begin
                                    nVal:= IdentList[eid].Offset;
                                    v:= DoCalc(val, RetOpr, v);
                                    nVal:= DoCalc(nVal, opr, v);
                                    AddCodeListValue(mType, nVal);
                                 end else begin
                                    nVal:= DoCalc(val, RetOpr, v);
                                    AddCodeListEQUOFFSET(mType, enam, opr, nVal);
                                 end;
                              end;
			                        if t = equModeSIZE then begin
                                 nFnd:= true;
                                 val:= IdentList[id].IValue;
                                 opr:= IdentList[id].EquOpr;
                                 eid:= IdentList[id].EquIdentIndex;
                                 enam:= IdentList[id].EquName;
                                 if eid <> NotFound then begin
                                    nVal:= IdentList[eid].Len;
                                    v:= DoCalc(val, RetOpr, v);
                                    nVal:= DoCalc(nVal, opr, v);
                                    AddCodeListValue(mType, nVal);
                                 end else begin
                                    nVal:= DoCalc(val, RetOpr, v);
                                    AddCodeListEQUSIZE(mType, enam, opr, nVal);
                                 end;
                              end;
			                        if not nFnd then begin
                                 ErrorMsg('Incompatible equate type [' + nam + ']!');
			                        end;
                           end;
                        end;
                     end else begin
                        v:= GetCalc;
                        if not IsErr then AddCodeListMEMID(mType, id, RetOpr, v);
                     end;
                  end;
                  {break;}
               end;
            end else begin
               if (GetSrcChar = '"') or (GetSrcChar = #39) then begin
                  GrabDataString(mType);
               end else begin
                  num:= GetNumber(mType);
                  if not IsErr then begin
                     AddCodeListValue(mType, num);
                  end;
               end;
            end;
         end;
         if IsErr then break;
         SkipBlank;
         if GetSrcChar = ',' then begin
            NextChar;
            SkipBlank;
         end else begin
            break;
         end;
      end;
   end;
   if not IsErr then begin
      if n <> '' then begin
         siz:= 1;
         if mType = tBYTE then siz:= 1;
         if mType = tWORD then siz:= 2;
         if mType = tDWORD then siz:= 4;
         if mType = tQWORD then siz:= 8;
         if mType = tFARPTR then siz:= 6;
         if mType = tTENBYTE then siz:= 10;
         AddMemory(n, remPos, mType, siz);
      end;
   end;
end;



procedure DefineArray(n: string);
var
 remPos            : TAsmValue;
 mType             : byte;
 siz               : TAsmValue;
 nn                : string;
 v                 : TAsmValue;
 i, cnt            : TDefValue;
begin
   remPos:= GetSegOffset(true);
   mType:= tNONE;
   nn:= GetName;
   cnt:= 1;
   if not IsErr then begin
      if nn <> '' then begin
         if nn = 'BYTE' then mType:= tBYTE;
         if nn = 'WORD' then mType:= tWORD;
         if nn = 'DWORD' then mType:= tDWORD;
         if nn = 'QWORD' then mType:= tQWORD;
         if nn = 'FARPTR' then mType:= tFARPTR;
         if nn = 'TENBYTE' then mType:= tTENBYTE;
         if mType <> tNONE then begin
            v:= GetNumber(mType);
            if not IsErr then begin
               SkipBlank;
               if GetSrcChar = ',' then begin
                  NextChar;
                  cnt:= GetNumber(tDWORD);
                  if not IsErr then begin
                     if cnt > 0 then begin
                        i:= 1;
                        while i <= cnt do begin
                           AddCodeListValue(mType, v);
                           Inc(i);
                        end;
                     end else begin
                        ErrorMsg('Array Count must be over 0!');
                     end;
                  end;
               end else begin
                  ErrorMsg('[ , ] expected!');
               end;
            end;
         end else begin
            ErrorMsg('Invalid Type [' + nn + ']!');
         end;
      end else begin
         ErrorMsg('Array Type expected!');
      end;
   end;

   if not IsErr then begin
      if n <> '' then begin
         siz:= 1;
         if mType = tBYTE then siz:= 1;
         if mType = tWORD then siz:= 2;
         if mType = tDWORD then siz:= 4;
         if mType = tQWORD then siz:= 8;
         if mType = tFARPTR then siz:= 6;
         if mType = tTENBYTE then siz:= 10;
         siz:= siz * cnt;
         AddMemory(n, remPos, mType, siz);
      end;
   end;
end;

end.
