{ Program models
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _model;

interface

uses
 _common;

type
 TModel = record
    Name           : string;
    SegTableOfs    : TDefValue;
    SegDataOfs     : TDefValue;
    SegmentCount   : TAsmValue;
 end;
 TSegData = record
    Bits           : TAsmValue;
    Limit          : TAsmValue;
    FillSize       : TAsmValue;
    RoundSize      : TAsmValue;
    WriteSEG       : boolean;
 end;

var
 ModelList         : array of TModel;
 ModelCount        : TDefValue;

 SegDataList       : array of TSegData;
 SegDataCount      : TDefValue;

procedure InitModels;
procedure AddModel(n: string);
procedure SetModel(n: string);

implementation

uses
 _error, _mem, _scanner, _parser, _seg;

procedure InitModels;
begin
   SetLength(ModelList, 1);
   ModelCount:= 0;
   SetLength(SegDataList, 1);
   SegDataCount:= 0;
end;

procedure AddModel(n: string);
var
 mFound            : boolean;
 i                 : TDefValue;
 stOf, sdOf        : TDefValue;
 sCnt              : TAsmValue;
 segN, wri         : string;
 sBits             : TAsmValue;
 sLimit            : TAsmValue;
 sFill             : TAsmValue;
 sRound            : TAsmValue;
 sWrite            : boolean;
 segIndx           : TDefValue;
begin
   mFound:= false;
   if ModelCount > 0 then begin
      i:= 0;
      while i < ModelCount do begin
         if n = ModelList[i].Name then begin
            mFound:= true;
            break;
         end;
         Inc(i);
      end;
   end;
   if not mFound then begin
      stOf:= MemLength;
      sdOf:= SegDataCount;
      sCnt:= 0;
      while true do begin
         SkipBlank;
         if GetSrcChar = '(' then begin
            NextChar;
            segN:= GetName;
            sBits:= 0;
            sLimit:= 0;
            sFill:= 0;
            sRound:= 0;
            sWrite:= true;
            if not IsErr then begin
               if segN <> '' then begin
                  sBits:= GetNumber(tAUTO);
                  if not IsErr then sLimit:= GetNumber(tAUTO);
                  if not IsErr then sFill:= GetNumber(tAUTO);
                  if not IsErr then sRound:= GetNumber(tAUTO);
                  if not IsErr then wri:= GetName;
                  if not IsErr then begin
                     if wri <> '' then begin
                        if (wri = 'ON') or (wri = 'OFF') then begin
                           if wri = 'ON' then begin
                              sWrite:= true;
                           end else begin
                              sWrite:= false;
                           end;
                        end else begin
                           ErrorMsg('[ON] or [OFF] expected, but [' + wri +
                            '] found!');
                        end;
                     end;
                  end;
               end else begin
                  ErrorMsg('Segment name expected!');
               end;
            end;
            if IsErr then break;
            SkipBlank;
            if GetSrcChar = ')' then begin
               NextChar;
               segIndx:= NotFound;
               i:= 0;
               while i < SegCount do begin
                  if segN = Segment[i].IdentName then begin
                     segIndx:= i;
                     break;
                  end;
                  Inc(i);
               end;
               if segIndx <> NotFound then begin
                  MemReplacePos:= -1;
                  AddMemValue(segIndx);
                  SetLength(SegDataList, SegDataCount + 1);
                  SegDataList[SegDataCount].Bits:= sBits;
                  SegDataList[SegDataCount].Limit:= sLimit;
                  SegDataList[SegDataCount].FillSize:= sFill;
                  SegDataList[SegDataCount].RoundSize:= sRound;
                  SegDataList[SegDataCount].WriteSEG:= sWrite;
                  Inc(SegDataCount);
                  Inc(sCnt);
               end else begin
                  ErrorMsg('Segment not found [' + segN + ']!');
                  break;
               end;
            end else begin
               ErrorMsg('[ ) ] expected!');
               break;
            end;
         end else begin
            break;
         end;
      end;
      if not IsErr then begin
         SetLength(ModelList, ModelCount + 1);
         ModelList[ModelCount].Name:= n;
         ModelList[ModelCount].SegTableOfs:= stOf;
         ModelList[ModelCount].SegDataOfs:= sdOf;
         ModelList[ModelCount].SegmentCount:= sCnt;
         Inc(ModelCount);
      end;
   end else begin
      ErrorMsg('Model already exists [' + n + ']!');
   end;
end;

procedure SetModel(n: string);
var
 modIndex, i       : TDefValue;
 stOfs             : TDefValue;
 sdOfs             : TDefValue;
 sCnt              : TAsmValue;
 segI              : TDefValue;
 sBits             : TAsmValue;
 sLimit            : TAsmValue;
 sFill             : TAsmValue;
 sRound            : TAsmValue;
 sWrite            : boolean;
begin
   modIndex:= NotFound;
   if ModelCount > 0 then begin
      i:= 0;
      while i < ModelCount do begin
         if n = ModelList[i].Name then begin
            modIndex:= i;
            break;
         end;
         Inc(i);
      end;
   end;
   if modIndex <> NotFound then begin
      stOfs:= ModelList[modIndex].SegTableOfs;
      sdOfs:= ModelList[modIndex].SegDataOfs;
      sCnt:= ModelList[modIndex].SegmentCount;
      if sCnt > 0 then begin
         i:= 0;
         while i < sCnt do begin
            segI:= MemData[stOfs + i];
            sBits:= SegDataList[sdOfs + i].Bits;
            sLimit:= SegDataList[sdOfs + i].Limit;
            sFill:= SegDataList[sdOfs + i].FillSize;
            sRound:= SegDataList[sdOfs + i].RoundSize;
            sWrite:= SegDataList[sdOfs + i].WriteSEG;
            Segment[segI].SegBits:= sBits;
            Segment[segI].Limit:= sLimit;
            Segment[segI].FillSize:= sFill;
            Segment[segI].RoundSize:= sRound;
            Segment[segI].WriteSEG:= sWrite;
            Inc(i);
         end;
      end;
   end else begin
      ErrorMsg('Model not found [' + n + ']!');
   end;
end;

end.
