{ Registers
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _reg;

interface

uses
 _common;

type
 TReg = record
    Name           : string;
    Code           : TAsmValue;
    SegValue       : TAsmValue;
 end;
 TRegTable = record
    Name           : string;
    RegOffset      : TDefValue;
    RegCount       : TAsmValue;
 end;

var
 Regs              : array of TReg;
 CurReg            : TDefValue;

 RegTable          : array of TRegTable;
 CurRegTable       : TDefValue;

procedure InitRegs;

function GetRegIndex(reg: string): TDefValue;
function GetRegTableIndex(regTbl: string): TDefValue;

procedure AddReg(rName: string; rCode, rSVal: TAsmValue);
procedure AddRegTable(tblName: string);

implementation

uses
 _error, _mem, _scanner, _parser;

procedure InitRegs;
begin
   SetLength(Regs, 1);
   CurReg:= 0;
   SetLength(RegTable, 1);
   CurRegTable:= 0;
end;



function GetRegIndex(reg: string): TDefValue;
var
 id, i             : TDefValue;
begin
   id:= NotFound;
   if CurReg > 0 then begin
      i:= 0;
      while i < CurReg do begin
         if Regs[i].Name = reg then begin
	          id:= i;
            break;
         end;
         Inc(i);
      end;
   end;
   GetRegIndex:= id;
end;

function GetRegTableIndex(regTbl: string): TDefValue;
var
 id, i             : TDefValue;
begin
   id:= NotFound;
   if CurRegTable > 0 then begin
      i:= 0;
      while i < CurRegTable do begin
         if RegTable[i].Name = regTbl then begin
	          id:= i;
            break;
         end;
         Inc(i);
      end;
   end;
   GetRegTableIndex:= id;
end;



procedure AddReg(rName: string; rCode, rSVal: TAsmValue);
begin
   if GetRegIndex(rName) = NotFound then begin
      SetLength(Regs, CurReg + 1);
      Regs[CurReg].Name:= rName;
      Regs[CurReg].Code:= rCode;
      Regs[CurReg].SegValue:= rSVal;
      Inc(CurReg);
   end else begin
      ErrorMsg('Register already exists [' + rName + ']!');
   end;
end;

procedure AddRegTable(tblName: string);
var
 rName             : string;
 regID, i          : TDefValue;
 regOfs            : TDefValue;
 regCnt            : TAsmValue;
begin
   if GetRegTableIndex(tblName) = NotFound then begin
      regOfs:= MemLength;
      regCnt:= 0;
      while true do begin
         rName:= GetName;
         if rName <> '' then begin
            regID:= NotFound;
            i:= 0;
            while i < CurReg do begin
               if Regs[i].Name = rName then begin
                  regID:= i;
                  break;
               end;
               Inc(i);
            end;
            if regID <> NotFound then begin
               MemReplacePos:= -1;
               AddMemValue(regID);
               Inc(regCnt);
            end else begin
               ErrorMsg('Register does not exists [' + rName + ']!');
               break;
            end;
         end else begin
            ErrorMsg('Register name expected!');
            break;
         end;
         SkipBlank;
         if GetSrcChar = ',' then begin
            NextChar;
         end else begin
            break;
         end;
      end;
      if not IsErr then begin
         SetLength(RegTable, CurRegTable + 1);
         RegTable[CurRegTable].Name:= tblName;
         RegTable[CurRegTable].RegOffset:= regOfs;
         RegTable[CurRegTable].RegCount:= regCnt;
         Inc(CurRegTable);
      end;
   end else begin
      ErrorMsg('Register Table already exists [' + tblName + ']!');
   end;
end;

end.
