{ Registers - PIE
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _reg2;

interface

const
 noReg = 8;



function IsReg8(r: string): boolean;
function IsReg16(r: string): boolean;
function IsReg32(r: string): boolean;
function IsStdReg(r: string): boolean;
function IsSegReg(r: string): boolean;

function IsCtrlReg(r: string): boolean;
function IsTestReg(r: string): boolean;
function IsDebugReg(r: string): boolean;

function IsAnyReg(r: string): boolean;
function GetRegCode(r: string): byte;

function GetModRM2(r1,r2: string; use32: boolean): byte;
function GetSpecialModRM2(reg,value: cardinal): byte;

function GetModRMDigit2(reg: string; digit: byte; use32: boolean): byte;

implementation

uses
 _error;

function IsReg8(r: string): boolean;
var
 iR8: boolean;
begin
   iR8:= false;
   if r = 'AL' then iR8:= true;
   if r = 'CL' then iR8:= true;
   if r = 'DL' then iR8:= true;
   if r = 'BL' then iR8:= true;
   if r = 'AH' then iR8:= true;
   if r = 'CH' then iR8:= true;
   if r = 'DH' then iR8:= true;
   if r = 'BH' then iR8:= true;
   IsReg8:= iR8;
end;

function IsReg16(r: string): boolean;
var
 iR16: boolean;
begin
   iR16:= false;
   if r = 'AX' then iR16:= true;
   if r = 'CX' then iR16:= true;
   if r = 'DX' then iR16:= true;
   if r = 'BX' then iR16:= true;
   if r = 'SI' then iR16:= true;
   if r = 'DI' then iR16:= true;
   if r = 'SP' then iR16:= true;
   if r = 'BP' then iR16:= true;
   IsReg16:= iR16;
end;

function IsReg32(r: string): boolean;
var
 iR32: boolean;
begin
   iR32:= false;
   if r = 'EAX' then iR32:= true;
   if r = 'ECX' then iR32:= true;
   if r = 'EDX' then iR32:= true;
   if r = 'EBX' then iR32:= true;
   if r = 'ESI' then iR32:= true;
   if r = 'EDI' then iR32:= true;
   if r = 'ESP' then iR32:= true;
   if r = 'EBP' then iR32:= true;
   IsReg32:= iR32;
end;

function IsStdReg(r: string): boolean;
begin
   IsStdReg:= (IsReg8(r)) or (IsReg16(r)) or (IsReg32(r));
end;

function IsSegReg(r: string): boolean;
var
 iSR: boolean;
begin
   iSR:= false;
   if r = 'CS' then iSR:= true;
   if r = 'DS' then iSR:= true;
   if r = 'ES' then iSR:= true;
   if r = 'FS' then iSR:= true;
   if r = 'GS' then iSR:= true;
   if r = 'SS' then iSR:= true;
   IsSegReg:= iSR;
end;



function IsCtrlReg(r: string): boolean;
var
 iCR: boolean;
begin
   iCR:= false;
   if r = 'CR0' then iCR:= true;
   if r = 'CR2' then iCR:= true;
   if r = 'CR3' then iCR:= true;
   if r = 'CR4' then iCR:= true;
   IsCtrlReg:= iCR;
end;

function IsTestReg(r: string): boolean;
var
 iTR: boolean;
begin
   iTR:= false;
   if r = 'TR3' then iTR:= true;
   if r = 'TR4' then iTR:= true;
   if r = 'TR5' then iTR:= true;
   if r = 'TR6' then iTR:= true;
   if r = 'TR7' then iTR:= true;
   IsTestReg:= iTR;
end;

function IsDebugReg(r: string): boolean;
var
 iDR: boolean;
begin
   iDR:= false;
   if r = 'DR0' then iDR:= true;
   if r = 'DR1' then iDR:= true;
   if r = 'DR2' then iDR:= true;
   if r = 'DR3' then iDR:= true;
   if r = 'DR6' then iDR:= true;
   if r = 'DR7' then iDR:= true;
   IsDebugReg:= iDR;
end;



function IsAnyReg(r: string): boolean;
begin
   IsAnyReg:= (IsStdReg(r)) or (IsSegReg(r)) or
    (IsCtrlReg(r)) or (IsTestReg(r)) or (IsDebugReg(r));
end;

function GetRegCode(r: string): byte;
var
 rCode: byte;
begin
   rCode:= noReg;
   if r = 'AL' then rCode:= 0;
   if r = 'CL' then rCode:= 1;
   if r = 'DL' then rCode:= 2;
   if r = 'BL' then rCode:= 3;
   if r = 'AH' then rCode:= 4;
   if r = 'CH' then rCode:= 5;
   if r = 'DH' then rCode:= 6;
   if r = 'BH' then rCode:= 7;
   if r = 'AX' then rCode:= 0;
   if r = 'CX' then rCode:= 1;
   if r = 'DX' then rCode:= 2;
   if r = 'BX' then rCode:= 3;
   if r = 'SP' then rCode:= 4;
   if r = 'BP' then rCode:= 5;
   if r = 'SI' then rCode:= 6;
   if r = 'DI' then rCode:= 7;
   if r = 'EAX' then rCode:= 0;
   if r = 'ECX' then rCode:= 1;
   if r = 'EDX' then rCode:= 2;
   if r = 'EBX' then rCode:= 3;
   if r = 'ESP' then rCode:= 4;
   if r = 'EBP' then rCode:= 5;
   if r = 'ESI' then rCode:= 6;
   if r = 'EDI' then rCode:= 7;
   if r = 'ES' then rCode:= 0;
   if r = 'CS' then rCode:= 1;
   if r = 'SS' then rCode:= 2;
   if r = 'DS' then rCode:= 3;
   if r = 'FS' then rCode:= 4;
   if r = 'GS' then rCode:= 5;
   if r = 'CR0' then rCode:= 0;
   if r = 'CR2' then rCode:= 2;
   if r = 'CR3' then rCode:= 3;
   if r = 'CR4' then rCode:= 4;
   if r = 'TR3' then rCode:= 3;
   if r = 'TR4' then rCode:= 4;
   if r = 'TR5' then rCode:= 5;
   if r = 'TR6' then rCode:= 6;
   if r = 'TR7' then rCode:= 7;
   if r = 'DR0' then rCode:= 0;
   if r = 'DR1' then rCode:= 1;
   if r = 'DR2' then rCode:= 2;
   if r = 'DR3' then rCode:= 3;
   if r = 'DR6' then rCode:= 6;
   if r = 'DR7' then rCode:= 7;
   GetRegCode:= rCode;
end;



function GetModRM2(r1,r2: string; use32: boolean): byte;
begin
   GetModRM2:= GetModRMDigit2(r1, GetRegCode(r2), use32);
end;

function GetSpecialModRM2(reg, value: cardinal): byte;
begin
   GetSpecialModRM2:= value + (reg * 8);
end;



function GetModRMDigit2(reg: string; digit: byte; use32: boolean): byte;
var
 b: byte;
begin
   if digit = noReg then begin
      if GetRegCode(reg) = noReg then begin
         { memory:memory }
         ErrorMsg('Invalid ModR/M combination (memory-to-memory)!');
         b:= 0;
      end else begin
         { reg:memory }
         if (use32) then begin
            b:= GetSpecialModRM2(GetRegCode(reg), 5);
         end else begin
            b:= GetSpecialModRM2(GetRegCode(reg), 6);
         end;
      end;
   end else begin
      if GetRegCode(reg) = noReg then begin
         { memory:reg }
         if (use32) then begin
            b:= GetSpecialModRM2(digit, 5);
         end else begin
            b:= GetSpecialModRM2(digit, 6);
         end;
      end else begin
         { reg:reg }
         if IsSegReg(reg) then begin
            b:= $c0 + (digit + (GetRegCode(reg) * 8));
         end else begin
            b:= $c0 + (GetRegCode(reg) + (digit * 8));
         end;
      end;
   end;
   GetModRMDigit2:= b;
end;

end.
