{ Sources
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _source;

interface

uses
 _common;

const
 srcFILE           = 0;
 srcMACRO          = 1;
 srcOBJECT         = 2;

 srcNONE           = 5;

type
 TState = record
    Cond           : boolean;
    StateType      : byte;
    StartLine      : TDefValue;
    Variable       : string;
    StartVal       : TAsmValue;
    EndVal         : TAsmValue;
    CurVal         : TAsmValue;
    Step           : TAsmValue;
 end;

 TSource = record
    SourceID       : TDefValue;
    FileName       : string;
    SrcType        : byte;
    SrcOffset      : TDefValue;
    CurLine        : TDefValue;
    NumLines       : TDefValue;
    SrcLine        : string;
    Pos            : integer;
    Len            : integer;
    ParamValOfs    : TDefValue;
    ParamValCnt    : TDefValue;
    StateLevel     : TDefValue;
    StateSkip      : TDefValue;
    BreakFound     : boolean;
    Statements     : array of TState;
    StateSkipTable : array of byte;
 end;
 TSourceTree = record
    Level          : TDefValue;
    SourceType     : integer;
    SourceOffset   : TDefValue;
    NumLines       : TDefValue;
    SourceName     : string;
 end;

var
 CurID             : TDefValue;
 SourceLines       : array of string;
 NumSourceLines    : TDefValue;

 Source            : array of TSource;
 NumSources        : TDefValue;
 CurLevel          : TDefValue;
 FileLevel         : TDefValue;

 SourceTree        : array of TSourceTree;
 SourceTreeLines   : TDefValue;

procedure InitSource;
procedure SetSource(indx, CurID: TDefValue; filName: string; sTyp: byte);
procedure ReadSource(srcID: TDefValue);

function IsEndLine: boolean;
function IsLastLine: boolean;

procedure AddSourceData(lev: TDefValue; srcType: integer;
 srcOffs, nmLines: TDefValue; srcName: string);

implementation

uses
 _error, _state, _emit;

procedure InitSource;
begin
   SetLength(SourceLines, 1);
   SetLength(Source, 1);
   SetLength(SourceTree, 1);

   CurID:= 0;
   NumSourceLines:= 0;

   NumSources:= 0;
   CurLevel:= 0;
   FileLevel:= 0;

   SourceTreeLines:= 0;
end;

procedure SetSource(indx, CurID: TDefValue; filName: string; sTyp: byte);
begin
   Source[indx].SourceID:= CurID;
   Source[indx].FileName:= filName;
   Source[indx].SrcType:= sTyp;
   Source[indx].SrcOffset:= NumSourceLines;
   Source[indx].CurLine:= 1;
   Source[indx].NumLines:= 0;
   Source[indx].SrcLine:= '';
   Source[indx].Pos:= 1;
   Source[indx].Len:= 0;
   Source[indx].ParamValOfs:= 0;
   Source[indx].ParamValCnt:= 0;
   Source[indx].StateLevel:= 0;
   Source[indx].StateSkip:= 0;
   Source[indx].BreakFound:= false;

   SetLength(Source[indx].Statements, 1);
   SetLength(Source[indx].StateSkipTable, 1);

   Source[indx].Statements[0].Cond:= true;
end;

procedure ReadSource(srcID: TDefValue);
var
 LineCnt           : TDefValue;
 oFile             : textfile;
 s                 : string;
begin
   LineCnt:= 0;
   {$I-}
   Assign(oFile, Source[srcID].FileName);
   Reset(oFile);
      while not eof(oFile) do begin
         ReadLn(oFile, s);
         SetLength(SourceLines, NumSourceLines + 1);
         SourceLines[NumSourceLines]:= s;
         Inc(NumSourceLines);
         Inc(LineCnt);
      end;
   Close(oFile);
   {$I+}
   if IOResult <> 0 then begin
      ErrorMsg('Can not access file ['+ Source[srcID].FileName + ']!');
   end;
   Source[srcID].NumLines:= LineCnt;
end;



function IsEndLine: boolean;
var
 iEL               : boolean;
begin
   iEL:= false;
   if Source[CurLevel].Pos > Source[CurLevel].Len then iEL:= true;
   IsEndLine:= iEL;
end;

function IsLastLine: boolean;
begin
   IsLastLine:= (Source[CurLevel].CurLine > Source[CurLevel].NumLines);
end;



procedure AddSourceData(lev: TDefValue; srcType: integer;
 srcOffs, nmLines: TDefValue; srcName: string);
begin
   SetLength(SourceTree, SourceTreeLines+1);
   SourceTree[SourceTreeLines].Level:= lev;
   SourceTree[SourceTreeLines].SourceType:= srcType;
   SourceTree[SourceTreeLines].SourceOffset:= srcOffs;
   SourceTree[SourceTreeLines].NumLines:= nmLines;
   SourceTree[SourceTreeLines].SourceName:= srcName;
   Inc(SourceTreeLines);
end;

end.
